/*
Rabih Kaddour
Copyright 2016! Inc. All rights reserved.
Licensed under the BSD License.
http://kaddour-es.de
*/

////////////////////////////
////// HELPERS
////////////////////////////
var renderer = null;
var scene = null;
var camera = null;
var canvas = null;
var fov = 75;
var near = 0.1;
var far = 2000;

function init() {
  // Get the document with Id
  var elmtDiv = document.getElementById('WebGLCanvas');
  if(elmtDiv) {
    canvas = elmtDiv;
    canvas.height = elmtDiv.clientHeight;
    canvas.width = elmtDiv.clientWidth;
  }
  else {
    canvas = document.getElementById('WebGLCanvas');
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
  }

  // Create a camera
  // Set a Field of View (FOV) of 75 degrees
  // Set an Apsect Ratio of the inner width divided by the inner height of the window
  // Set the 'Near' distance at which the camera will start rendering scene objects to 2
  // Set the 'Far' (draw distance) at which objects will not be rendered to 1000
  camera = new THREE.PerspectiveCamera(fov, canvas.width / canvas.height, near, far);


  // Set the camera position so that it's up top and looking down:
  var posX=200, posY=200, posZ=350;
  // var deg = 90 * Math.PI / 180; // degInRad
  // camera.lookAt(new THREE.Vector3(0,0,0));
  // camera.rotateOnAxis(new THREE.Vector3(1, 0, 0), (deg));
  // camera.up = new THREE.Vector3(1, 0, 0);
  camera.position.z = posZ;
  /*
  var posX=200, posY=200, posZ=250;
  var deg = 90 * Math.PI / 180; // degInRad
  camera.lookAt(new THREE.Vector3(0,0,0));
  camera.rotateOnAxis(new THREE.Vector3(1, 0, 0), (deg));
  camera.up = new THREE.Vector3(0, 0, 1);
  // Move the camera 'out' by 200, 200 and 250
  camera.position.set(posX, posY, posZ);
  */

  // Create main scene
  // This is the scene we will add all objects to.
  // Create a scene, that will hold all our elements such as objects, cameras and lights.
  scene = new THREE.Scene();

  // White directional light at half intensity shining from the top.
  var directionalLight = new THREE.DirectionalLight( 0xffffff, 1 );
  scene.add( directionalLight );

  // Add the camera to the scene
  scene.add(camera);

  // Add cube to scene
  AddCube();

  // Add cylinder to scene
  AddCylinder();

  // Add torus to scene
  addTorus(50, 7);

  // Add sphere to scene
  addSphere(37);

  // camera.lookAt(scene.position);

  // Create a WebGL Rendered
  renderer = new THREE.WebGLRenderer();

  // Set the size of the rendered to the inner width and inner height of the window
  renderer.setPixelRatio( window.devicePixelRatio );
  renderer.setSize(canvas.width, canvas.height);

  // Background
  renderer.setClearColor(0xdddddd); // gray

  // Add in the created DOM element to the body of the document
  canvas.appendChild(renderer.domElement);

  // callback rendering
  // render();
  requestAnimationFrame(render);

  // Register an event listener to
  // call the resizeCanvas() function each time
  // the window is resized.
  window.addEventListener('resize', resizeRendererToDisplaySize, false);
}

// var material = new THREE.MeshLambertMaterial({color: 0xff3300});

// Create a cube
function AddCube() {
  var material = new THREE.MeshNormalMaterial({ opacity: 0.1 });
  var cubeGeo = new THREE.BoxGeometry(77, 77, 77);
  var cube = new THREE.Mesh(cubeGeo, material);
  // cube.rotateZ(-Math.PI / 3);
  cube.rotateY(-Math.PI / 3);
  cube.position.set(-30,0,0);

  // Add the cube to the scene
  scene.add(cube);
}

// Create a cylinder
function AddCylinder() {
  var material = new THREE.MeshNormalMaterial({ opacity: 0.1 });
  var cylGeometry = new THREE.CylinderGeometry(30, 30, 77, 64, false);
  var cylinder = new THREE.Mesh(cylGeometry, material);
  // cylinder.rotateX(-Math.PI / 2);
  cylinder.position.set(87,0,0);

  // Add the cylinder to the scene
  scene.add(cylinder);
}

function addTorus(radiusMajor, radiusMinor) {
  try {
      var torusGeometry = new THREE.TorusGeometry(radiusMajor, radiusMinor, 16, 100);
      var material = new THREE.MeshNormalMaterial({ opacity: 0.1 });
      var torusMesh = new THREE.Mesh(torusGeometry, material);
      torusMesh.rotateX(-Math.PI / 3);
      torusMesh.position.set(-157, 0, 0);
      // torusMesh.position.set(0, 0, (radiusMajor - radiusMinor) / 2);
      torusMesh.name = "Torus";

      // Add the torus to the scene
      scene.add(torusMesh);
  } catch (error) {
      // TODO: handle exception
      var msg = "Error to make the torus: " + error.toString();
      alert(msg);
      throw Error(msg);
      return false;
  }
}

function addSphere(sphRadius) {
  try {
      var sphereGeometry = new THREE.SphereGeometry(sphRadius, 32, 32);
      var material = new THREE.MeshNormalMaterial({ opacity: 0.1 });
      var sphereMesh = new THREE.Mesh(sphereGeometry, material);
      sphereMesh.rotateX(-Math.PI / 2);
      sphereMesh.position.set(197, 0, 0);
      sphereMesh.name = "Sphere";

      // Add the axes to the scene
      scene.add(sphereMesh);

  } catch (error) {
      // TODO: handle exception
      var msg = "Error to make sphere: " + error.toString();
      alert(msg);
      throw Error(msg);
      return false;
  }
}

// Animate the scene
function render(time) {
    // Render the scene
    renderScene(time);

    // This function calls itself on every frame. You can for example change
    // the objects rotation on every call to create a turntable animation.
    requestAnimationFrame( render );
}

// var clock = new THREE.Clock();
function renderScene(time) {
  // time *= 0.001;
  //
  // cubes.forEach((cube, ndx) => {
  //   const speed = .2 + ndx * .1;
  //   const rot = time * speed;
  //   cube.rotation.x = rot;
  //   cube.rotation.y = rot;
  // });

	renderer.render( scene, camera );
};

// Runs each time the DOM window resize event fires.
// Resets the canvas dimensions to match window,
// then draws the new borders accordingly.
function resizeRendererToDisplaySize() {
    // Check if the canvas is not the same size.
    var elmtDiv = document.getElementById('WebGLCanvas'); // img with id
    if(elmtDiv)
    {
    	canvas = elmtDiv;
    	canvas.height = elmtDiv.clientHeight;
    	canvas.width = elmtDiv.clientWidth;
    }
    if(camera != null)
    {
        camera.aspect = canvas.width / canvas.height;
        camera.updateProjectionMatrix();
    }
    if(renderer != null)
        renderer.setSize( canvas.width, canvas.height );
}
